#
# Simple PyChips interactive register access tool
#
"""
Eric Hazen
Updated by Charlie Hill
05/17/2012
"""
from PyChipsUser import *
import readline
import sys
import amc13

def help():
  print "R <addr>                read address (one word)"
  print "R <addr> <count>        read address <count> words"
  print "W <addr> <data>         write one word"
  print "RB <addr> <count>       block read"
  print "WB <addr> <count>       block write (test data 0...<count>-1)"
  print "RF <addr> <count>       FIFO read (non-incrementing)"
  print "V                       Switch to Virtex"
  print "S                       Switch to Spartan"
  print "Q                       quit"

print "IPbus interactive debugger"

sn = amc13.sn1()
t1, t2 = amc13.connect(sn)
t1_ip, t2_ip = amc13.ip_addr(sn)                                                                                                                                                                                  
print "Using IP address %s for Virtex" %t1_ip
print "Using IP address %s for Spartan" %t2_ip

while True:
  print "Type 'v' for Virtex Chip or 's' for Spartan Chip"
  k = raw_input('>> ')
  m = k.upper()
  if m == 'V':
    print "Now using Virtex Chip"
    x = t1
    break
  elif m == 'S':
    print "Now using Spartan Chip"
    x = t2
    break
  else:
    True

help()

while 1:
  s = raw_input('>> ')
  a = s.split()
  if len(a) >= 1:
    cmd = a[0].upper()

    if cmd == 'R':
      # see if first argument is a register name or a value (hex/decimal)
      if a[1].isdigit() or a[1].startswith("0x"):
        base = "ID"
        addr = int(a[1], 0)
      else:
        base = a[1]
        addr = 0
      try:
        v = x.read(base,addr)
        if len(a) == 3:
          n = int(a[2], 0)
          for i in range(n):
            v = x.read(base,addr+i)
            print "0x%04x: " % (addr+i), " 0x%08x" % v
        else:
          print "0x%04x: " % addr, " 0x%08x" % v
      except ChipsException:
        print "Given addr does not exist in AddrTable"
           
    elif cmd == 'W':
      if a[1].isdigit() or a[1].startswith("0x"):
        base = "ID"
        addr = int(a[1], 0)
      else:
        base = a[1]
        addr = 0
      if len(a) == 3:
        try:
          d = int( a[2], 0)
          x.write( base, d, addr)
        except ChipsException:
          print "Given addr does not exist in AddrTable"
      else:
        print "Need address and write value after W"
        
    elif cmd == 'RB':
      if a[1].isdigit() or a[1].startswith("0x"):
        base = "ID"
        addr = int(a[1], 0)
      else:
        base = a[1]
        addr = 0
      if len(a) == 3:
        n = int( a[2], 0)
        print "Block read", n, "words from", addr
        try:
          block = x.blockRead(base, n, addr)
          for i in xrange( n):
            print "%04x: " % (addr+i), " 0x%08x" % block[i]
        except ChipsException:
          print "Either given addr does not exist in AddrTable"
          print "Or given addr is masked in AddrTable, which is not allowed"
      else:
        print "Need address and count after RB"

    elif cmd == 'RF':
      if a[1].isdigit() or a[1].startswith("0x"):
        base = "ID"
        addr = int(a[1], 0)
      else:
        base = a[1]
        addr = 0
      if len(a) == 3:
        n = int( a[2], 0)
        print "FIFO read", n, "words from", addr
        try:
          block = x.fifoRead(base, n, addr)
          for i in xrange( n):
            print "%04x: " % (addr+i), " 0x%08x" % block[i]
        except ChipsException:
          print "Given addr does not exist in AddrTable"
      else:
        print "Need address and count after RF"

    elif cmd == "WB":
      if a[1].isdigit() or a[1].startswith("0x"):
        base = "ID"
        addr = int(a[1], 0)
      else:
        base = a[1]
        addr = 0
      if len(a) == 3:
          n = int( a[2], 0)
          print "Block write", n, "words to", addr
          block = []
          for i in xrange( n):
            block.append( i & 0xff)
          try:
            x.blockWrite(base, block, addr)
          except ChipsException:
            print "Either given addr does not exist in AddrTable"
            print "Or given addr is masked in AddrTable, which is not allowed"
      else:
        print "Need address and count after WB"

    elif cmd == "WF":
      if a[1].isdigit() or a[1].startswith("0x"):
        base = "ID"
        addr = int(a[1], 0)
      else:
        base = a[1]
        addr = 0
      if len(a) == 3:
          n = int( a[2], 0)
          print "FIFO write", n, "words to", addr
          block = []
          for i in xrange( n):
            block.append( i & 0xff)
          try:
            x.fifoWrite(base, block, addr)
          except ChipsException:
            print "Given addr does not exist in AddrTable"
      else:
        print "Need address and count after WF"

    elif cmd == "V":
      x = t1
      print "Now using the Virtex Chip"

    elif cmd == "S":
      x = t2
      print "Now using the Spartan Chip"

    elif cmd == "Q":
      sys.exit(0)

    else:
      help()
    
